package net.pingex.dcf.permissions;

import net.pingex.dcf.commands.Command;
import sx.blah.discord.handle.obj.IGuild;
import sx.blah.discord.handle.obj.IMessage;
import sx.blah.discord.handle.obj.IRole;
import sx.blah.discord.handle.obj.IUser;

import java.util.Collections;
import java.util.List;

/**
 * Landing class of this package.
 */
public class PermissionsHandler
{
	private static final ICommandPermissionsProvider CURRENT_PROVIDER = new DefaultPermissionsProvider();

	/**
	 * Tells whether someone can run requested command.
	 *
	 * The validator will successively test the following until something other than `null` is returned.
	 * - Individual user permission on the called guild using {@code ICommandPermissionsProvider.validateUser()}
	 * - Individual user permission globally using {@code ICommandPermissionsProvider.validateUser()}
	 * - The user's role list using {@code ICommandPermissionsProvider.validateGroup()}
	 * - Default behavior included in a Command object
	 *
	 * @param request Request message, includes author, guild, and so on.
	 * @param command Requested command, as parsed by {@code CommandHandler}
	 * @return `true` if request is granted, `false` if denied.
	 */
	public static boolean canRunCommand(IMessage request, Command command)
	{
		IUser requestAuthor = request.getAuthor();
		IGuild originatedGuild;
		List<IRole> rolesForUser;
		try
		{
			originatedGuild = request.getGuild();
			rolesForUser = requestAuthor.getRolesForGuild(originatedGuild);
		}
		catch(UnsupportedOperationException uoe) // DM
		{
			originatedGuild = null;
			rolesForUser = Collections.emptyList();
		}

		// First check: user permission for guild, skipped if DM
		if(originatedGuild != null)
		{
			Boolean canUserGuildRun = CURRENT_PROVIDER.validateUser(originatedGuild, requestAuthor, command);
			if(canUserGuildRun != null) return canUserGuildRun;
		}

		// Second check: user permission globally
		Boolean canUserRun = CURRENT_PROVIDER.validateUser(null, requestAuthor, command);
		if(canUserRun != null) return canUserRun;

		// Third check: user role permissions, auto-skipped if DM
		for(IRole i : rolesForUser)
		{
			Boolean canRoleRun = CURRENT_PROVIDER.validateGroup(i, command);
			if(canRoleRun != null) return canRoleRun;
		}

		// Fourth check: default behavior
		return command.getDefaultPermission().test(request);
	}
}
